import _extends from 'babel-runtime/helpers/extends';
import _defineProperty from 'babel-runtime/helpers/defineProperty';
import _objectWithoutProperties from 'babel-runtime/helpers/objectWithoutProperties';
import _classCallCheck from 'babel-runtime/helpers/classCallCheck';
import _possibleConstructorReturn from 'babel-runtime/helpers/possibleConstructorReturn';
import _inherits from 'babel-runtime/helpers/inherits';
import React, { Component } from 'react';
import * as PropTypes from 'prop-types';
var classNames = require('classnames');

function noop() {}

var Switch = function (_Component) {
  _inherits(Switch, _Component);

  function Switch(props) {
    _classCallCheck(this, Switch);

    var _this = _possibleConstructorReturn(this, _Component.call(this, props));

    _initialiseProps.call(_this);

    var checked = false;
    if ('checked' in props) {
      checked = !!props.checked;
      // modified by njq.niu@hand-china.com
    } else if ('value' in props) {
      checked = props.checkedValue === props.value;
    } else {
      checked = !!props.defaultChecked;
    }
    _this.state = { checked: checked };
    return _this;
  }

  Switch.prototype.componentDidMount = function componentDidMount() {
    var _props = this.props,
        autoFocus = _props.autoFocus,
        disabled = _props.disabled;

    if (autoFocus && !disabled) {
      this.focus();
    }
  };

  Switch.prototype.componentWillReceiveProps = function componentWillReceiveProps(nextProps) {
    if ('checked' in nextProps) {
      this.setState({
        checked: !!nextProps.checked
      });
      // modified by njq.niu@hand-china.com
    } else if ('value' in nextProps) {
      this.setState({
        checked: nextProps.checkedValue === nextProps.value
      });
    }
  };

  Switch.prototype.setChecked = function setChecked(checked) {
    if (this.props.disabled) {
      return;
    }
    if (!('checked' in this.props)) {
      this.setState({
        checked: checked
      });
    }

    // modified by njq.niu@hand-china.com
    var _props2 = this.props,
        _props2$checkedValue = _props2.checkedValue,
        checkedValue = _props2$checkedValue === undefined ? true : _props2$checkedValue,
        _props2$unCheckedValu = _props2.unCheckedValue,
        unCheckedValue = _props2$unCheckedValu === undefined ? false : _props2$unCheckedValu;

    var value = checked ? checkedValue : unCheckedValue;
    this.props.onChange(value);
  };

  // Handle auto focus when click switch in Chrome


  Switch.prototype.focus = function focus() {
    this.node.focus();
  };

  Switch.prototype.blur = function blur() {
    this.node.blur();
  };

  Switch.prototype.render = function render() {
    var _classNames;

    var _props3 = this.props,
        className = _props3.className,
        prefixCls = _props3.prefixCls,
        disabled = _props3.disabled,
        loadingIcon = _props3.loadingIcon,
        checkedValue = _props3.checkedValue,
        unCheckedValue = _props3.unCheckedValue,
        checkedChildren = _props3.checkedChildren,
        tabIndex = _props3.tabIndex,
        unCheckedChildren = _props3.unCheckedChildren,
        restProps = _objectWithoutProperties(_props3, ['className', 'prefixCls', 'disabled', 'loadingIcon', 'checkedValue', 'unCheckedValue', 'checkedChildren', 'tabIndex', 'unCheckedChildren']);

    var checked = this.state.checked;
    var switchTabIndex = disabled ? -1 : tabIndex || 0;
    var switchClassName = classNames((_classNames = {}, _defineProperty(_classNames, className, !!className), _defineProperty(_classNames, prefixCls, true), _defineProperty(_classNames, prefixCls + '-checked', checked), _defineProperty(_classNames, prefixCls + '-disabled', disabled), _classNames));
    return React.createElement(
      'span',
      _extends({}, restProps, {
        className: switchClassName,
        tabIndex: switchTabIndex,
        ref: this.saveNode,
        onKeyDown: this.handleKeyDown,
        onClick: this.toggle,
        onMouseUp: this.handleMouseUp
      }),
      loadingIcon,
      React.createElement(
        'span',
        { className: prefixCls + '-inner' },
        checked ? checkedChildren : unCheckedChildren
      )
    );
  };

  return Switch;
}(Component);

var _initialiseProps = function _initialiseProps() {
  var _this2 = this;

  this.toggle = function () {
    var onClick = _this2.props.onClick;

    var checked = !_this2.state.checked;
    _this2.setChecked(checked);
    onClick(checked);
  };

  this.handleKeyDown = function (e) {
    if (e.keyCode === 37) {
      // Left
      _this2.setChecked(false);
    } else if (e.keyCode === 39) {
      // Right
      _this2.setChecked(true);
    } else if (e.keyCode === 32 || e.keyCode === 13) {
      // Space, Enter
      _this2.toggle();
    }
  };

  this.handleMouseUp = function (e) {
    if (_this2.node) {
      _this2.node.blur();
    }
    if (_this2.props.onMouseUp) {
      _this2.props.onMouseUp(e);
    }
  };

  this.saveNode = function (node) {
    _this2.node = node;
  };
};

Switch.propTypes = {
  className: PropTypes.string,
  prefixCls: PropTypes.string,
  disabled: PropTypes.bool,
  checkedChildren: PropTypes.any,
  unCheckedChildren: PropTypes.any,
  onChange: PropTypes.func,
  onMouseUp: PropTypes.func,
  onClick: PropTypes.func,
  tabIndex: PropTypes.number,
  checked: PropTypes.bool,
  defaultChecked: PropTypes.bool,
  autoFocus: PropTypes.bool,
  loadingIcon: PropTypes.node,
  checkedValue: PropTypes.any, // modified by njq.niu@hand-china.com
  unCheckedValue: PropTypes.any // modified by njq.niu@hand-china.com
};

Switch.defaultProps = {
  prefixCls: 'rc-switch',
  checkedChildren: null,
  unCheckedChildren: null,
  className: '',
  defaultChecked: false,
  checkedValue: true, // modified by njq.niu@hand-china.com
  unCheckedValue: false, // modified by njq.niu@hand-china.com
  onChange: noop,
  onClick: noop
};

export default Switch;