import * as React from 'react';
import RcTable from '../rc-components/table';
import * as PropTypes from 'prop-types';
import { Store } from './createStore';
import Column from './Column';
import ColumnGroup from './ColumnGroup';
import { SpinProps } from '../spin';
import { TableProps, TableSize, TableState, TableComponents, RowSelectionType, TableLocale, ColumnProps, TableStateFilters, SelectionItemSelectFn, SelectionInfo, CustomColumn, ExportProps, handleProps } from './interface';
import { RadioChangeEvent } from '../radio';
import { CheckboxChangeEvent } from '../checkbox';
import { ConfigConsumerProps } from '../config-provider';
export default class Table<T> extends React.Component<TableProps<T>, TableState<T>, ConfigConsumerProps> {
    static TableRowContext: React.Context<any>;
    static Column: typeof Column;
    static ColumnGroup: typeof ColumnGroup;
    static propTypes: {
        dataSource: PropTypes.Requireable<any[]>;
        columns: PropTypes.Requireable<any[]>;
        prefixCls: PropTypes.Requireable<string>;
        useFixedHeader: PropTypes.Requireable<boolean>;
        resizable: PropTypes.Requireable<boolean>;
        rowSelection: PropTypes.Requireable<object>;
        className: PropTypes.Requireable<string>;
        size: PropTypes.Requireable<string>;
        loading: PropTypes.Requireable<boolean | object>;
        bordered: PropTypes.Requireable<boolean>;
        onChange: PropTypes.Requireable<(...args: any[]) => any>;
        locale: PropTypes.Requireable<object>;
        dropdownPrefixCls: PropTypes.Requireable<string>;
        exported: PropTypes.Requireable<object>;
        autoHeight: PropTypes.Requireable<boolean | object>;
    };
    static defaultProps: {
        dataSource: never[];
        prefixCls: string;
        useFixedHeader: boolean;
        className: string;
        size: TableSize;
        loading: boolean;
        bordered: boolean;
        resizable: boolean;
        indentSize: number;
        locale: {};
        rowKey: string;
        showHeader: boolean;
        autoHeight: boolean;
    };
    CheckboxPropsCache: {
        [key: string]: any;
    };
    store: Store;
    columns: ColumnProps<T>[];
    components: TableComponents;
    row: React.ComponentType<any>;
    iframe: HTMLIFrameElement;
    element: HTMLDivElement;
    wrapper: HTMLDivElement;
    constructor(props: TableProps<T>);
    setElementRef: (node: RcTable) => void;
    setWapperRef: (node: HTMLDivElement) => void;
    getCheckboxPropsByItem: (item: T, index: number) => any;
    getDefaultSelection(): any[];
    getDefaultPagination(props: TableProps<T>): {};
    componentWillReceiveProps(nextProps: TableProps<T>): void;
    onRow: (record: T, index: number) => any;
    setSelectedRowKeys(selectedRowKeys: string[], selectionInfo: SelectionInfo<T>): void;
    hasPagination(props?: any): boolean;
    isFiltersChanged(filters: TableStateFilters): boolean;
    getSortOrderColumns(columns?: ColumnProps<T>[]): any;
    getFilteredValueColumns(columns?: ColumnProps<T>[]): any;
    getFiltersFromColumns(columns?: ColumnProps<T>[]): any;
    getDefaultSortOrder(columns?: ColumnProps<T>[]): {
        sortColumn: any;
        sortOrder: any;
    };
    getSortStateFromColumns(columns?: ColumnProps<T>[]): {
        sortColumn: any;
        sortOrder: any;
    };
    getSorterFn(): ((a: T, b: T) => number) | undefined;
    toggleSortOrder(order: 'ascend' | 'descend', column: ColumnProps<T>): void;
    handleFilter: (column: ColumnProps<T>, nextFilters: string[]) => void;
    handleSelect: (record: T, rowIndex: number, e: CheckboxChangeEvent) => void;
    handleRadioSelect: (record: T, rowIndex: number, e: RadioChangeEvent) => void;
    handleSelectRow: (selectionKey: string, index: number, onSelectFunc: SelectionItemSelectFn) => any;
    handlePageChange: (current: number, ...otherArguments: any[]) => void;
    renderSelectionBox: (type: RowSelectionType | undefined) => (_: any, record: T, index: number) => JSX.Element;
    getRecordKey: (record: T, index: number) => any;
    getPopupContainer: () => HTMLElement;
    renderRowSelection(columns: ColumnProps<T>[], locale: TableLocale): ColumnProps<T>[];
    getColumnKey(column: ColumnProps<T>, index?: number): string | number | undefined;
    getMaxCurrent(total: number): number | undefined;
    isSortColumn(column: ColumnProps<T>): boolean;
    renderColumnsDropdown(columns: ColumnProps<T>[], locale: TableLocale): any[];
    setCustomState(customColumns: any[]): void;
    renderCustomColumns: (columns: ColumnProps<T>[]) => ColumnProps<T>[];
    /**
     * 获取用户个性化的 Table 的 头设置
     * table's columns
     */
    getCustomColumns: () => (CustomColumn[] | undefined);
    /**
     * 用户个性化 Table 编辑
     */
    handleCustomColumnFilter: () => void;
    handleShowSizeChange: (current: number, pageSize: number) => void;
    renderPagination(paginationPosition: string): JSX.Element | null;
    prepareParamsArguments(state: any): [any, string[], Object];
    findColumn(myKey: string | number): undefined;
    componentDidMount(): void;
    getCurrentPageData(): T[];
    getFlatData(): any[];
    getFlatCurrentPageData(): any[];
    recursiveSort(data: T[], sorterFn: (a: any, b: any) => number): T[];
    getLocalData(): T[];
    createComponents(components?: TableComponents, prevComponents?: TableComponents): void;
    handleResize: (col: ColumnProps<T>) => (_: any, dragCallbackData: any) => void;
    getContentHeight: () => number;
    syncSize: () => number;
    renderTable: (contextLocale: TableLocale, loading: SpinProps) => JSX.Element;
    renderConfigConsumer(): (configProps: ConfigConsumerProps) => "" | JSX.Element;
    renderExportButton: (props: ExportProps) => JSX.Element;
    handleExport: (exportedProps: handleProps) => void;
    render(): JSX.Element;
}
